/*
Copyright © 2025 Philipp Wolfer <phw@uploadedlobster.com>

This file is part of Scotty.

Scotty is free software: you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later version.

Scotty is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
Scotty. If not, see <https://www.gnu.org/licenses/>.
*/

package models

import (
	"errors"
	"iter"

	"github.com/simonfrey/jsonl"
	"go.uploadedlobster.com/scotty/pkg/archive"
)

type JSONLFile[T any] struct {
	File archive.OpenableFile
}

func (f *JSONLFile[T]) openReader() (*jsonl.Reader, error) {
	if f.File == nil {
		return nil, errors.New("file not set")
	}
	fio, err := f.File.Open()
	if err != nil {
		return nil, err
	}
	reader := jsonl.NewReader(fio)
	return &reader, nil
}

func (f *JSONLFile[T]) IterItems() iter.Seq2[T, error] {
	return func(yield func(T, error) bool) {
		reader, err := f.openReader()
		if err != nil {
			var listen T
			yield(listen, err)
			return
		}
		defer reader.Close()

		for {
			var listen T
			err := reader.ReadSingleLine(&listen)
			if err != nil {
				break
			}
			if !yield(listen, nil) {
				break
			}
		}
	}
}
